@@ -15,11 +15,13 @@ GC            <- 102L
 DEBUG         <- 103L
 EVAL          <- 104L
 SET           <- 105L
-GET           <- 106L
-GET_REFERENCE <- 107L
-DEF           <- 108L
-CLEAR         <- 109L
-INVOKE        <- 110L
+SET_SINGLE    <- 106L
+SET_DOUBLE    <- 107L
+GET           <- 108L
+GET_REFERENCE <- 109L
+DEF           <- 110L
+INVOKE        <- 111L
+SCALAP        <- 112L
 OK <- 1000L
 ERROR <- 1001L
 UNDEFINED_IDENTIFIER <- 1002L
