@@ -688,7 +688,7 @@ object RClient {
     val portsFile = File.createTempFile("rscala-","")
     val processInstance = processCmd.run(processIO)
     val snippet = s"""
-rscala:::rServe(rscala:::newSockets('${portsFile.getAbsolutePath.replaceAll(File.separator,"/")}',debug=${if ( debug ) "TRUE" else "FALSE"},timeout=${timeout}))
+rscala:::rServe(rscala:::newSockets('${portsFile.getAbsolutePath.replace(File.separator,"/")}',debug=${if ( debug ) "TRUE" else "FALSE"},timeout=${timeout}))
 q(save='no')
     """
     while ( cmd == null ) Thread.sleep(100)
